% This program uses SP500 option prices (as proxied by the ETF SPY) to
% obtain parameter estimates for the Variance Gamma model of Madan, Carr,
% and Chang (1998) 
% "The Variance Gamma Process and Option Pricing"
% European Finance Review, Vol. 2, pp. 79-105
% Implementation is from Madan and Milne (1991)
% "Option Pricing with V.G. Martingale Components"
% Mathematical Finance, Vol. 1, No. 4, pp. 39-55

% Fabrice Douglas Rouah, FRouah.com and Volopta.com

clc; clear;

%% Read in call market prices and strikes for SPY, Long Version
data = xlsread('SPY Options', 'Prices', 'A2:B69');
K  = data(:,1);
MarketPrice = data(:,2);
n    = length(K);

% Spot, maturity, risk free rate, dividend yield
Spot = 90.692;
T = 0.194387;
r = 0.0179;
q = 0;

%% Define starting values.
start = [0 .25 .25];
options = optimset('MaxFunEvals',1e10, 'MaxIter', 1e6, 'TolX', 1e-5);

%% Run the optimization
[params feval] = fminsearch(@(b) fRMSE(b, Spot, K, MarketPrice, r, q, T), start, options);

%% Use estimated parameters to obtain fitted Variance Gamma call prices
% Expression for call price is W on page 50 of Madan and Milne (1991)
alpha = params(1);
sigma = params(2);
v     = params(3);

for i=1:n
    VGPrice(i) = fVGCall(alpha,sigma,v,Spot,K(i),r,T);
end

%% Fit Black Scholes price using historical volatility
returns = xlsread('SPY Options', 'Returns', 'C2:C83');
vol = sqrt(252)*std(returns);

BSCall = @(s,K,r,q,v,T) s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T));
for i=1:n
    BSPrice(i) = BSCall(Spot,K(i),r,q,T,vol);
end;

%% Output everything
fprintf('Parameter             Value  \n')
fprintf('-----------------------------\n')
fprintf('alpha             %10.4f \n',alpha)
fprintf('sigma             %10.4f \n',sigma)
fprintf('v                 %10.4f \n',v)
fprintf('historical vol    %10.4f \n',vol)
fprintf('objective funct   %10.4f \n',feval)
fprintf('-----------------------------\n')

fprintf('Strike  MarketPrice    VGPrice      BSPrice \n')
fprintf('--------------------------------------------\n')
for i=1:n
    fprintf('%4.0f %12.4f %12.4f %12.4f \n',K(i),MarketPrice(i),VGPrice(i),BSPrice(i));
end
fprintf('--------------------------------------------\n')



